
/*
 * $Id: Client.cc,v 1.1.1.4 1999/02/07 20:21:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Rnd.h"
#include "Connection.h"
#include "StatPhaseMgr.h"
#include "SharedOpts.h"
#include "CltOpts.h"
#include "CltXact.h"
#include "Client.h"

#include "globals.h"

Client::Client(): theConnMgr(0), theThinkDistr(0), theLaunchCount(0) {
}

Client::~Client() {
	// XXX: assumes clients are destroyed only when no alarms will ring
	if (thePendAlarmCnt)
		thePendAlarmCnt = 0; // prevent AlarmUser::dtor from complaining
}

void Client::configure(CltConnMgr *aConnMgr, int ccLevel, RndDistr *aDistr) {
	theConnMgr = aConnMgr;
	theXacts.limit(ccLevel);
	theThinkDistr = aDistr;
}

void Client::start() {
	Assert(!theLaunchCount);
	TheStatPhaseMgr->lock(); // wait until we launch a xaction

	if (TheCltOpts.theLaunchWin > 0) {
		static RndGen *rng = new RndGen(TheOpts.theRngSeed);
		RndDistr *launchDistr = new UnifDistr(rng, 0, TheCltOpts.theLaunchWin.secd());
		sleepFor(Time::Secd(launchDistr->trial()));
	} else
		scheduleLaunch(TheClock);
}

void Client::noteXactDone(CltXact *x) {
	Assert(x);
	theConnMgr->put(x->conn());
	theXacts.put(x);
}

void Client::noteConnReady(Connection *conn) {
	if (!theLaunchCount++)
		TheStatPhaseMgr->unlock();
	CltXact *x = theXacts.get();
	x->exec(this, conn);
}

void Client::scheduleLaunch(Time lastLaunch) {
	Assert(!thePendAlarmCnt);
	if (theThinkDistr)
		sleepTill(lastLaunch + Time::Secd(theThinkDistr->trial()));
	else
		launch();
}

void Client::launch() {
	theConnMgr->order(this);
}
