#include "polygraph.h"

/*
 * $Id: CltConnMgr.cc,v 1.1.1.4 1999/02/22 06:19:47 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Connection.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "CltOpts.h"
#include "CltConnMgr.h"
#include "polyErrors.h"
#include "gadgets.h"
#include "globals.h"

CltConnMgr::CltConnMgr(): theCcGoal(-1), thePendLevel(0), theUseLevel(0) {
}

CltConnMgr::~CltConnMgr() {
	while (theReadyConns.count()) delete theReadyConns.dequeue();
}

void CltConnMgr::configure(const NetAddr &anAddr, int aCcGoal, int ccLimit) {
	theCcGoal = aCcGoal; // but may be adjusted later
	theAddr = anAddr;
	const int max = Max(1024, 1 + 4*Max(0, ccLimit, theCcGoal));
	theConns.limit(max);
	theReadyConns.resize(max);
	theUsers.resize(1+ccLimit);
	theConnIdx.stretch(Socket::TheMaxLevel+Socket::TheMaxLevel/2);
}

void CltConnMgr::start() {
	maintainCcLevel();
}

void CltConnMgr::maintainCcLevel() {
	while ((behindDemand() || behindGoal()) && open());
}

void CltConnMgr::checkCcLevel() {
	if (TheCltOpts.adjustConnPoolSize) {
		if (theUsers.count())
			theCcGoal++;
		else
		if (thePendLevel && theCcGoal > theUseLevel)
			theCcGoal--;
	}
	maintainCcLevel();
}

void CltConnMgr::order(User *u) {
	if (theReadyConns.count()) {
		give(theReadyConns.dequeue(), u);
	} else {
		theUsers.enqueue(u);
		maintainCcLevel();
	}
}

void CltConnMgr::put(Connection *conn) {
	Assert(conn);
	// XXX: no persistent connections
	theConnIdx[conn->sock().fd()] = 0;
	conn->close();
	theUseLevel--;
	theConns.put(conn);
	maintainCcLevel();
}

void CltConnMgr::give(Connection *conn, User *u) {
	Assert(u);
	theUseLevel++;
	u->noteConnReady(conn);
}

Connection *CltConnMgr::open() {
	Connection *conn = theConns.get();

	const int attemptLimit = 16;
	for (int i = attemptLimit; i > 0; --i) {
		if (conn->connect(theAddr))
			return open2(conn);
		conn->reset();
	}
	if (!thePendLevel) {
		cerr << here << "failure to connect after " << attemptLimit << " attempts with no pending connections" << endl;
		FatalError(errno);
	}
	return 0;
}

Connection *CltConnMgr::open2(Connection *conn) {
	const int fd = conn->sock().fd();
	Assert(!theConnIdx[fd]);
	theConnIdx[fd] = conn;

	thePendLevel++;

	/*cerr << here
		<< "connecting.. fd: " << setw(3) << fd 
		<< " ord: " << setw(3) << theUsers.count()
		<< " ccl: " << setw(3) << ccLevel() 
		<< " pf: " << setw(3) << prefetchLevel()
		<< endl;*/

	conn->theWr.theReserv = TheSelect->setFD(fd, dirWrite, this);
	return conn;
}

void CltConnMgr::noteWriteReady(int fd) {
	Connection *conn = theConnIdx[fd];
	Assert(conn && conn->sock().fd() == fd);
	thePendLevel--;
	CallStat(ThePinger, connEstb());
	if (theUsers.count()) {
		give(conn, theUsers.dequeue());
	} else {
		TheSelect->clearFD(conn->theWr.theReserv);
		theReadyConns.enqueue(conn);
	}
	maintainCcLevel();
}
