#include "polygraph.h"

/*
 * $Id: Http.cc,v 1.1.1.2 1999/02/19 07:30:50 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */


#include "Http.h"
#include "gadgets.h"


void HttpHdr::reset() {
	theHdrSize = 0;
	theContSize = -1;
	theXactId.clear();

	theSrchBuf = 0;
	theSrchPos = 0;
	theSrchState = ssFirst;
}

// note: buf may not be 0-terminated!
bool HttpHdr::parse(const char *buf, Size sz) {
	if (!theSrchBuf) { // have not started the search yet
		theSrchBuf = buf;
		theSrchPos = 0;
		theSrchState = ssFirst;
	} else {           // continue search
		Assert(theSrchBuf == buf);
		Assert(theSrchState != ssFound);
	}

	buf += theSrchPos;
	while (theSrchState != ssFound && theSrchPos < sz) {
		if (theSrchState == ssFirst) {
			for (; theSrchPos < sz && *buf != '\n'; ++theSrchPos, ++buf);
			if (theSrchPos < sz)
				theSrchState = ssSkip;
		} else
		if (theSrchState == ssSkip) {
			for (; theSrchPos < sz && *buf == '\r'; ++theSrchPos, ++buf);
			if (theSrchPos < sz)
				theSrchState = (*buf == '\n') ? ssFound : ssFirst;
		}
		if (theSrchPos < sz) {
			theSrchPos++;
			buf++;
		}
	}

	if (theSrchState != ssFound)
		return false;

	// found end-of-headers!
	theHdrSize = theSrchPos;

	// second pass: parse known fields
	theSrchPos = 0;
	while (theSrchPos < theHdrSize) {
		parseField();
		// skip until next field
		for (; theSrchPos < theHdrSize && *theSrchBuf != '\n'; ++theSrchPos, ++theSrchBuf);
		if (theSrchPos < theHdrSize) {
			theSrchPos++;
			theSrchBuf++;
		}
	}
	return true;
}

bool HttpHdr::parseField() {
	if (fieldEq("Content-length:", 15))
		theContSize = xatoi(theSrchBuf, -1);
	else
	if (fieldEq("X-Xact:", 7)) {
		if (const char *b = strchr(theSrchBuf, '.'))
			if (const char *c = strchr(++b, ':'))
				theXactId = UniqId(xatoi(theSrchBuf), xatoi(b), xatoi(c+1));
	} else
		return false;
	return true;
}

// compares current field with name and increments search position on match
// note: since field names do not contain '\n', and we already found
//       end-of-headers, strcmp is safe to use
bool HttpHdr::fieldEq(const char *name, Size len) {
	if (strncasecmp(theSrchBuf, name, len) == 0) {
		theSrchPos += len;
		theSrchBuf += len;
		return true;
	}
	return false;
}


/* ReqHdr */

void ReqHdr::reset() {
	HttpHdr::reset();
	theObjId = theETag = -1;
}

bool ReqHdr::parseField() {
	if (HttpHdr::parseField())
		return true;
	else
	if (fieldEq("GET ", 4)) {
		if (const char *h = strchr(theSrchBuf, '_'))
			theObjId = xatoi(h+1, -1);
	} else
	if (fieldEq("ETag:", 5)) {
		if (const char *h = strchr(theSrchBuf, '"'))
			theETag = xatoi(h+1, -1);
	} else
		return false;
	return true;
}

/* RepHdr */

void RepHdr::reset() {
	HttpHdr::reset();
	isCachable = true;
}

bool RepHdr::parseField() {
	if (HttpHdr::parseField())
		return true;
	else
	if (fieldEq("Pragma: no-cache", 16)) {
		isCachable = false;
	} else
		return false;
	return true;
}

