
/*
 * $Id: IOBuf.cc,v 1.1.2.1 1999/02/07 05:08:43 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Farm.h"
#include "LogComment.h"
#include "IOBuf.h"
#include "gadgets.h"

class BufPool: public Farm<char> {
	public:
		BufPool(Size aBufCap = -1);
		virtual ~BufPool() { while (!empty()) destroy(get()); } // g++ bug: {}

		void bufCap(Size aBufCap);
		Size bufCap() const { return theBufCap; }

	protected:
		virtual char *gen();
		virtual void destroy(char *b) { delete[] b; }
		virtual void overflow(char *b) { push(b); }

	protected:
		Size theBufCap;
		Size theRepLvl; // when to report growth
};

static BufPool *TheBufPool = 0;

/* IOBuf */


IOBuf::IOBuf(): theBuf(0), theCapacity(0), theOutOff(0), theInOff(0) {
	Assert(TheBufPool);
	theCapacity = TheBufPool->bufCap();
	Assert(theCapacity > 0);
	unzip();
}

void IOBuf::pack() {
	if (empty()) {
		reset();
	} else
	if (theOutOff) {
		memcpy(theBuf, content(), contSize());
		theInOff -= theOutOff;
		theOutOff = 0;
	}
}

// return [empty] buffer to shared area
void IOBuf::zip() {
	if (theBuf) { // could be zipped already
		TheBufPool->put(theBuf);
		theBuf = 0;
	}
}

void IOBuf::unzip() {
	if (!theBuf) {
		theBuf = TheBufPool->get();
		Assert(theBuf);
	}
}

/* WrBuf */

void WrBuf::append(const char *buf, Size sz) {
	Assert(spaceSize() >= sz);
	memcpy(space(), buf, sz);
	appended(sz);
}


/* BufPool */

BufPool::BufPool(Size aBufCap):
	theBufCap(aBufCap), theRepLvl(Size::MB(10)) {
}

void BufPool::bufCap(Size aBufCap) {
	Assert(empty() && !level());
	theBufCap = aBufCap;
}

char *BufPool::gen() {
	Assert(theBufCap > 0);
	char *b = new char[theBufCap];

	if (level()*theBufCap >= theRepLvl) {
		Comment(1) << "buffer pool grew to " 
			<< level() << " x " << theBufCap << " = " << (theBufCap*level()) << endc;
		theRepLvl = level()*theBufCap + Size::MB(10);
	}
	return b;
}


/* initialization */

int IOBufInit::theUseCount = 0;

void IOBufInit::init() {
	Assert(theUseCount == 1);
	TheBufPool = new BufPool(16*1024); // XXX: hardcoded buf capacity
}

void IOBufInit::clean() {
	Assert(theUseCount == 0);
	delete TheBufPool;
	TheBufPool = 0;
}

