
/*
 * $Id: OLog.cc,v 1.1.1.2 1999/02/20 05:43:51 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <string.h>

#include "OLog.h"
#include "logTags.h"
#include "gadgets.h"


OLog::OLog(): theStream(0), theBuf(0) {
	theCapacity = Size::MB(1); // default, to start with
	theBuf = new char[theCapacity];
	theSize = 0;
	theEntry = 0;
	putHeader();
}

OLog::~OLog() {
	if (theSize && theStream)
		flush();
	delete[] theBuf;
	delete theStream;
}

void OLog::stream(const String &aFileName, ostream *aStream) {
	theFileName = aFileName;
	theStream = aStream;
}

void OLog::resize(Size aCap) {
	Assert(!theEntry); // no logging is in progress
	char *buf = new char[theCapacity = aCap];
	if (theSize)
		memcpy(buf, theBuf, theSize);
	delete[] theBuf;
	theBuf = buf;
}

void OLog::close() {
	if (theStream)
		flush();
	delete theStream;
	theStream = 0;
}

void OLog::flush() {
	Assert(theStream);
	// do not flush current entry if any; its size is unknown
	const Size off = theEntry ? (Size)(theEntry - theBuf) : theSize;
	if (theSize) {
		Must(theStream->write(theBuf, off));
		theStream->flush();
		if (theSize > off) {
			memmove(theBuf, theBuf+off, theSize - off);
			theSize -= off;
			theEntry = theBuf;
		} else {
			theSize = 0;
		}
	}
}

void OLog::begEntry(int tag) {
	Assert(tag > 0);
	Assert(!theEntry);
	theEntry = theBuf + theSize;
	puti(0);		// size placeholder
	puti(tag);
}

void OLog::endEntry() {
	Assert(theEntry);
	const int x = htonl((theBuf + theSize) - theEntry);
	memcpy(theEntry, &x, sizeof(x)); // record actual entry size
	theEntry = 0;
}

void OLog::putHeader() {
	const int pxPoly1 = 0x506f6c79;
	const int pxPoly2 = 0x4c6f672e;
	puti(pxPoly1); puti(pxPoly2); puti(0); // magic
	puti(1); // current version
	puti(1); // required min version to support
	puti(0); // size of extra headers
	StoreSymbTable(*this);
}

void OLog::puti(const int *xs, int count) {
	puti(count);
	for (int i = 0; i < count; ++i)
		puti(xs[i]);
}
