
/*
 * $Id: StatPing.cc,v 1.1.1.7 1999/02/22 02:51:01 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iomanip.h>

#include "Clock.h"
#include "Socket.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "Progress.h"
#include "ILog.h"
#include "LogComment.h"
#include "logTags.h"
#include "gadgets.h"
#include "globals.h"

/* HRStat */

HRStat::HRStat() {
	reset();
}

void HRStat::reset() {
	theHitCnt = theMissCnt = 0;
	theHitSz = theMissSz = 0;
}

OLog &HRStat::store(OLog &log) const {
	return log << theHitCnt << theMissCnt << theHitSz << theMissSz;
}

ILog &HRStat::load(ILog &log) {
	return log >> theHitCnt >> theMissCnt >> theHitSz >> theMissSz;
}

double HRStat::dhr() const {
	return Ratio(theHitCnt, theHitCnt + theMissCnt);
}

double HRStat::bhr() const {
	return Ratio(theHitSz, theHitSz + theMissSz);
}

double HRStat::dhp() const {
	return Percent(theHitCnt, theHitCnt + theMissCnt);
}

double HRStat::bhp() const {
	return Percent(theHitSz, theHitSz + theMissSz);
}

/* StatPingRec */

StatPingRec::StatPingRec() {
	theSelCnt = theRdCnt = theWrCnt = 0;
	theRdSz = theWrSz = 0;
	theErrCnt = 0;
}

OLog &StatPingRec::store(OLog &log) const {
	Assert(theDuration >= 0);
	return log
		<< theXactLvl << theOpenLvl << theEstbLvl
		<< theSelCnt << theRdCnt << theWrCnt
		<< theRdSz << theWrSz
		<< theRealHR
		<< theErrCnt
		<< theDuration
		// XXX: << theIdealHR
		;
}

ILog &StatPingRec::load(ILog &log) {
	return log
		>> theXactLvl >> theOpenLvl >> theEstbLvl
		>> theSelCnt >> theRdCnt >> theWrCnt
		>> theRdSz >> theWrSz
		>> theRealHR
		>> theErrCnt
		>> theDuration
		// XXX: >> theIdealHR
		;
}

void StatPingRec::restart() {
	theXactLvl.restart();
	theOpenLvl.restart();
	theEstbLvl.restart();
	theSelCnt = theRdCnt = theWrCnt = 0;
	theRdSz = theWrSz = 0;
	theRealHR.reset();
	theIdealHR.reset();
	theChbR.reset();
	theErrCnt = 0;
}


/* StatPing */

StatPing::StatPing(Time aPeriod): StatClockInt(aPeriod) {
}

void StatPing::flush() {
	theDuration = TheClock - theIntStart;

	OLog &log = TheLog();
	log << bege(lgStatPing);
	StatPingRec::store(log);
	theIdealHR.store(log); // XXX
	theChbR.store(log); // XXX
	log << ende;

	if (theDuration > 0)
		report();
}

void StatPing::restart() {
	StatClockInt::restart();
	StatPingRec::restart();
}

void StatPing::report() const {
	Comment(2)
		<< "i-" << TheStatPhaseMgr->name()
		<< ' ' << setw(6) << TheProgress.theXactComplCnt
		<< ' ' << setw(6) << Ratio(theXactLvl.decCnt(), theDuration.secd())
		<< ' ' << setw(6) << theRealHR.dhp()
		<< ' ' << setw(3) << theErrCnt
		<< ' ' << setw(3) << Socket::Level()
		<< endc;
}

