
/*
 * $Id: Future.h,v 1.1.2.1 1999/02/22 03:31:33 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef FUTURE_CLASS_H
#define FUTURE_CLASS_H

#include "Array.h"

// "Future plan": a circular buffer; new entries are inserted
// at random positions relative to "now"; "now" or "next step"
// is at position "zero"
// for simplicity, assume that at most MAX_INT items are inserted
// over the life time of an object
template <class Item>
class Future {
	public:
		Future(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }
		int capacity() { return theLog.capacity(); }

		const Item &operator [](int off) const { return theLog[(theOffset + off) % theLog.capacity()]; }
		Item &operator [](int off) { return theLog[(theOffset + off) % theLog.capacity()]; }
		void shift(Item i) { theLog[theOffset++ % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // "now" or the start of a plan
};

#endif
