
/*
 * $Id: Heap.h,v 1.1.2.1 1999/01/30 17:42:53 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_HEAP_H
#define X_HEAP_H

#include "Array.h"

// Heap "tree" adopted with modifications from
// "Algorithms in C", Sedgewick, Addison-Wesley, 1990, ISBN 0-201-51425-7

template <class Item>
class Heap: protected Array<Item> {
	public:
		Heap(int aCapacity = 2): Array<Item>(aCapacity), theCnt(0) { theCount = 1; }

		int count() const { return theCnt; }
		bool empty() const { return theCnt <= 0; }

		Item &top() { return theItems[1]; }

		void add(Item v) { append(v); float_up(++theCnt); }
		void skip() { theItems[1] = theItems[theCnt--]; theCount--; sink_down(1); }
		Item shift() { const Item v = theItems[1]; skip(); return v; }

		void resize(int aCap) { Array<Item>::stretch(aCap); }

	protected:
		inline void float_up(int k);
		inline void sink_down(int k);

	protected:
		int theCnt;  // theCount-1
};


/* implementaion of in-lined methods */

template <class Item>
inline
void Heap<Item>::float_up(int k) {
	const Item v = theItems[k];
	for (int p = k/2; p && theItems[p] > v; k = p, p /= 2)
		theItems[k] = theItems[p];
	theItems[k] = v;
}

template <class Item>
inline
void Heap<Item>::sink_down(int k) {
	const int half = theCnt/2;
	const Item v = theItems[k];
	while (k <= half) {
		// smallest of the (at most) two kids
		int j = k*2;
		if (j < theCnt && theItems[j+1] < theItems[j])
			j++;
		if (v < theItems[j])
			break;
		theItems[k] = theItems[j];
		k = j;
	}
	theItems[k] = v;
}

#endif
